@@ -123,5 +123,5 @@ def parse_bed(bed, window):
     logging.info("Parsing BED file")
     gr = pr.read_bed(bed)[window.chromosome, window.begin:window.end]
     df = gr.unstrand().df
-    df = df.drop(columns=["Chromosome", "Score"])
+    df = df.drop(columns=["Chromosome", "Score", "Strand"], errors='ignore')
     return df.itertuples(index=False, name=None)
